<?php defined('SYSPATH') || die('No direct script access.');

class Controller_Student_Discipline extends Controller_Environment_Student
{
    public function action_show_subject() {
        $id = $this->request->param('id');
        $discipline = Model_Discipline::load($id);
        $rates = Model_Rating::getRates($this->user->RecordBookID, $id);

        $map = self::generateDisciplineMapFrom($rates);
        
        $this->twig->set([
            'Discipline'    => $discipline,
            'Teachers'      => self::getTeachersListOf($discipline),
            'DisciplineMap' => $map['Modules'],
            'Submodules'    => $map['Submodules'],
            'ExtraRate'     => $map['ExtraRate'],
            'Semester'      => Model_Semesters::load($discipline->SemesterID),
        ])->set_filename(static::TWIG_FOLDER . 'discipline/subject');
    }

    public function action_show_journal() {
        $id = $this->request->param('id');
        $discipline = Model_Discipline::load($id);

        $attendances = Model_Journal::getAttendanceForStudent($id, $this->user->RecordBookID);
        $lessonTypes = Model_Journal::getLessonTypes();
        $lessons = Model_Journal::getLessonsForDiscipline($id);

        $journal = [];
        foreach($lessons as $key => $lesson){
            $attended = false;
            foreach($attendances as $attendance) {
                if ($lesson["ID"] == $attendance["LessonID"]) {
                   if ($attendance["Attendance"])
                       $attended = true;
                   break;
                }
            }
            $journal[]=[
                "LessonDate" => $lesson["LessonDate"],
                "LessonType" => self::getLessonType($lesson["LessonType"], $lessonTypes),
                "Attended" => $attended
                ];
        }

        $this->twig->set([
            'Discipline'  => $discipline,
            'Teachers'      => self::getTeachersListOf($discipline),
            'Semester'      => Model_Semesters::load($discipline->SemesterID),
            'Journal'  => $journal
        ])->set_filename(static::TWIG_FOLDER . 'discipline/journal');
    }

    private static function getLessonType($key, $lessonTypes) {
        foreach ($lessonTypes as $type) {
            if ($key == $type["ID"]){
                return $type["LessonType"];
            }
        }
    }

    private static function &getTeachersListOf(Model_Discipline $discipline) {
        $teachers = $discipline->getTeachers()->as_array();

        foreach ($teachers as &$t) {
            $name = [ $t['LastName'], $t['FirstName'], $t['SecondName'] ];
            $t['Name'] = trim(implode($name, ' '));
        }

        return $teachers;
    }

    private static function generateDisciplineMapFrom(Database_Result $rates) {
        $map = [];                 # not sure, what it is about :(
        $extraRate = 0;            # completely legacy code, baby
        $submodules = [];
        $modules = [];

        foreach ($rates as $row) {
            $submodules[$row['SubmoduleID']] = [
                'Title'   => $row['SubmoduleName'],
                'MaxRate' => $row['MaxRate'],
                'Rate'    => $row['Rate'],
                'Date'    => $row['Date'],
            ];
            
            switch ($row['ModuleType']) {
                case 'exam':
                    $map['Exam'] = $row['SubmoduleID'];
                    break;

                case 'bonus':
                    $map['Bonus'] = $row['SubmoduleID'];
                    break;

                case 'extra':
                    $extraRate += $row['Rate'];
                    break;

                default:
                    $module =& $modules[$row['ModuleID']];
                    $module['Title'] = $row['ModuleName'];
                    $module['Submodules'][] = $row['SubmoduleID'];
            }
        }

        foreach ($submodules as &$s) {
            if ($s['ModuleType'] === 'extra')
                $extraRate += $s['Rate'];
        }

        $map['Modules'] = $modules;

        return ['Modules' => $map, 'Submodules' => $submodules, 'ExtraRate' => $extraRate];
    }
}
